import random
import sqlite3
import os
import json
import time
import requests
from flask import Flask, request, render_template, jsonify
from werkzeug.utils import secure_filename
from dotenv import load_dotenv
from threading import Thread
import logging

# تحميل متغيرات البيئة
load_dotenv()

app = Flask(__name__)

# إعداد متغيرات النظام
DB_FILE = "scheduled_posts.db"
PAGES_FILE = "pages.json"
UPLOAD_FOLDER = "uploads"
os.makedirs(UPLOAD_FOLDER, exist_ok=True)
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER

# إعداد قاعدة البيانات
def init_db():
    with sqlite3.connect(DB_FILE) as conn:
        cursor = conn.cursor()
        cursor.execute("""
        CREATE TABLE IF NOT EXISTS posts (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            message TEXT,
            pages TEXT,
            media TEXT,
            schedule_time INTEGER,
            status TEXT,
            token TEXT  -- إضافة عمود لتخزين التوكن
        )
        """)
        conn.commit()

init_db()

# جلب جميع التوكنات من ملف البيئة
def get_access_tokens():
    tokens = []
    i = 1
    while True:
        token = os.getenv(f"USER_ACCESS_TOKEN_{i}")
        if token:
            tokens.append(token)
            i += 1
        else:
            break
    return tokens if tokens else [os.getenv("USER_ACCESS_TOKEN")]  # الرجوع إلى التوكن الافتراضي إذا لم يكن هناك توكنات متعددة

# جلب Access Token للصفحات باستخدام توكن عشوائي
def get_all_page_access_tokens(access_token=None):
    all_pages = []
    tokens = get_access_tokens()  # نفترض أن هذه تعيد قائمة بالتوكنات

    if not access_token:
        access_token = random.choice(tokens)

    url = "https://graph.facebook.com/v18.0/me/accounts"
    params = {"access_token": access_token}

    attempt = 0
    max_attempts = len(tokens)

    while attempt < max_attempts:
        try:
            while url:  # استمر طالما يوجد رابط للصفحة التالية
                response = requests.get(url, params=params)
                data = response.json()

                if "data" in data:
                    all_pages.extend(data["data"])  # أضف بيانات الصفحة الحالية
                    logging.info(f"تم جلب الصفحات بنجاح باستخدام التوكن: {access_token[:10]}...")
                    # تحقق مما إذا كان هناك صفحة تالية
                    url = data.get("paging", {}).get("next", None)
                    params = {}  # إعادة تعيين params لأن 'next' يحتوي على كل المعلمات اللازمة
                else:
                    logging.error(f"فشل جلب الصفحات: {data}")
                    attempt += 1
                    if attempt < max_attempts:
                        access_token = tokens[attempt]  # جرب توكن آخر
                        url = "https://graph.facebook.com/v18.0/me/accounts"  # إعادة تعيين الـ URL
                        params = {"access_token": access_token}
                    else:
                        url = None  # لا توكنات أخرى، توقف
                    break

            if all_pages:  # إذا تم جلب بعض الصفحات بنجاح، ارجع
                break

        except Exception as e:
            logging.error(f"خطأ أثناء جلب الصفحات باستخدام {access_token[:10]}...: {str(e)}")
            attempt += 1
            if attempt < max_attempts:
                access_token = tokens[attempt]
                url = "https://graph.facebook.com/v18.0/me/accounts"
                params = {"access_token": access_token}
            else:
                break

    return all_pages

def get_page_access_token(page_id, access_token=None):
    all_pages = get_all_page_access_tokens(access_token)
    
    for page in all_pages:
        if page["id"] == page_id:
            return page["access_token"]
    return None

# الصفحة الرئيسية
@app.route('/')
def home():
    return render_template("index.html")

# API لجلب الصفحات
@app.route('/get_pages')
def get_pages():
    try:
        with open(PAGES_FILE, "r", encoding="utf-8") as f:
            pages = json.load(f)

        return jsonify({"pages": pages}) if pages else jsonify({"error": "❌ لا توجد صفحات"})

    except Exception as e:
        return jsonify({"error": str(e)})

# API لحفظ/نشر المنشورات
@app.route('/post', methods=['POST'])
def post_to_facebook():
    try:
        data = request.form
        message = data.get("message", "").strip()
        selected_pages = request.form.getlist("pages")
        schedule_time = data.get("schedule_time")
        action = data.get("action")
        schedule_interval = int(data.get("schedule_interval", 15))
        media_files = request.files.getlist("media")

        # حفظ الملفات
        saved_media_paths = []
        for media in media_files:
            if media.filename:
                filename = secure_filename(media.filename)
                file_path = os.path.join(app.config['UPLOAD_FOLDER'], filename)
                media.save(file_path)
                saved_media_paths.append(file_path)

        # جلب قائمة التوكنات
        tokens = get_access_tokens()

        # جدولة المنشورات
        if action == "schedule" and schedule_time:
            base_time = int(time.mktime(time.strptime(schedule_time, "%Y-%m-%dT%H:%M")))
            delay_per_page = schedule_interval * 60

            with sqlite3.connect(DB_FILE) as conn:
                cursor = conn.cursor()
                for index, page_id in enumerate(selected_pages):
                    page_schedule_time = base_time + (index * delay_per_page)
                    selected_token = random.choice(tokens)  # اختيار توكن عشوائي لكل صفحة

                    cursor.execute(
                        "SELECT COUNT(*) FROM posts WHERE message=? AND pages=? AND schedule_time=?",
                        (message, json.dumps([page_id]), page_schedule_time)
                    )
                    if cursor.fetchone()[0] == 0:
                        cursor.execute(
                            "INSERT INTO posts (message, pages, media, schedule_time, status, token) VALUES (?, ?, ?, ?, ?, ?)",
                            (message, json.dumps([page_id]), json.dumps(saved_media_paths), page_schedule_time, "scheduled", selected_token)
                        )
                conn.commit()
            return jsonify({"status": "scheduled", "message": f"✅ تم الجدولة بفارق {schedule_interval} دقيقة!"})

        # نشر فوري
        elif action == "publish":
            response_data = {}
            for page_id in selected_pages:
                selected_token = random.choice(tokens)  # اختيار توكن عشوائي لكل صفحة
                page_access_token = get_page_access_token(page_id, selected_token)
                if not page_access_token:
                    response_data[page_id] = {"error": "❌ رمز وصول غير صالح"}
                    continue

                media_ids = []
                for media in saved_media_paths:
                    try:
                        with open(media, "rb") as file:
                            media_response = requests.post(
                                f"https://graph.facebook.com/v18.0/{page_id}/photos",
                                files={"source": file},
                                data={"access_token": page_access_token, "published": "false"}
                            )
                            media_json = media_response.json()
                            media_ids.append({"media_fbid": media_json["id"]}) if "id" in media_json else None
                    except Exception as e:
                        response_data[page_id] = {"error": f"❌ خطأ في الملف: {e}"}
                        continue

                payload = {"message": message, "access_token": page_access_token}
                if media_ids:
                    payload["attached_media"] = json.dumps(media_ids)

                try:
                    response = requests.post(f"https://graph.facebook.com/v18.0/{page_id}/feed", data=payload)
                    response_data[page_id] = response.json()
                except Exception as e:
                    response_data[page_id] = {"error": str(e)}

            return jsonify(response_data)

        return jsonify({"status": "error", "message": "إجراء غير معروف!"})

    except Exception as e:
        return jsonify({"status": "error", "message": str(e)})

# API لجلب المنشورات المجدولة
@app.route('/get_scheduled_posts')
def get_scheduled_posts():
    try:
        with sqlite3.connect(DB_FILE) as conn:
            cursor = conn.cursor()
            cursor.execute("SELECT * FROM posts WHERE status = 'scheduled' ORDER BY schedule_time ASC")
            scheduled_posts = cursor.fetchall()

        posts_list = []
        for row in scheduled_posts:
            # التأكد من عدد الأعمدة بما يتناسب مع الجدول المحدث
            if len(row) == 7:  # مع وجود عمود token
                post_id, message, pages, media, schedule_time, status, token = row
            else:  # الجدول القديم بدون token
                post_id, message, pages, media, schedule_time, status = row
                token = None

            page_names = []
            try:
                pages = json.loads(pages)
                for page_id in pages:
                    page_name = get_page_name_by_id(page_id)
                    page_names.append(page_name)
            except json.JSONDecodeError:
                page_names = ["خطأ في تحليل الصفحات"]

            try:
                media_list = json.loads(media)
            except json.JSONDecodeError:
                media_list = []

            posts_list.append({
                "id": post_id,
                "message": message,
                "pages": page_names,
                "media": media_list,
                "schedule_time": schedule_time,
                "status": status,
                "token": token  # اختياري: يمكن إضافته للتحقق
            })

        return jsonify(posts_list)

    except Exception as e:
        return jsonify({"status": "error", "message": f"خطأ في جلب المنشورات: {str(e)}"})

def get_page_name_by_id(page_id):
    # هنا نستخدم `page_id` للحصول على اسم الصفحة من قاعدة البيانات أو من أي مصدر آخر
    # مثال على جلب اسم الصفحة من ملف JSON أو من قاعدة بيانات أخرى
    with open(PAGES_FILE, "r", encoding="utf-8") as f:
        pages = json.load(f)

    for page in pages:
        if page["page_id"] == page_id:
            return page["name"]  # إرجاع اسم الصفحة
    return "Unknown Page"

# حذف المنشورات
@app.route('/delete_post', methods=['POST'])
def delete_scheduled_post():
    try:
        post_id = request.form.get("post_id")
        if not post_id:
            return jsonify({"status": "error", "message": "❌ لم يتم تحديد المنشور"})

        with sqlite3.connect(DB_FILE) as conn:
            conn.execute("DELETE FROM posts WHERE id = ?", (post_id,))
        return jsonify({"status": "success", "message": "✅ تم الحذف بنجاح!"})

    except Exception as e:
        return jsonify({"status": "error", "message": str(e)})

@app.route('/delete_multiple_posts', methods=['POST'])
def delete_multiple_posts():
    try:
        post_ids = request.form.get("post_ids")

        if not post_ids:
            return jsonify({"status": "error", "message": "❌ لم يتم تحديد أي منشور للحذف."})

        post_ids = json.loads(post_ids)  # تحويل البيانات من JSON إلى قائمة

        with sqlite3.connect(DB_FILE) as conn:
            cursor = conn.cursor()
            cursor.execute("DELETE FROM posts WHERE id IN ({})".format(",".join("?" * len(post_ids))), post_ids)
            conn.commit()

        return jsonify({"status": "success", "message": f"✅ تم حذف {len(post_ids)} منشورات بنجاح!"})

    except Exception as e:
        return jsonify({"status": "error", "message": str(e)})

# معالجة النشر المجدول
is_thread_running = False

def publish_scheduled_posts():
    global is_thread_running
    
    if is_thread_running:
        return
    
    is_thread_running = True
    logging.basicConfig(filename='publish_logs.txt', level=logging.INFO)

    try:
        while True:
            with sqlite3.connect(DB_FILE) as conn:
                conn.execute("BEGIN IMMEDIATE")
                cursor = conn.cursor()
                current_time = int(time.time())
                
                cursor.execute(
                    "SELECT id, message, pages, media, token FROM posts "
                    "WHERE status = 'scheduled' AND schedule_time <= ? "
                    "ORDER BY schedule_time ASC LIMIT 1",
                    (current_time,)
                )
                post = cursor.fetchone()

                if not post:
                    conn.commit()
                    time.sleep(60)
                    continue

                post_id, message, pages_json, media_json, selected_token = post
                pages = json.loads(pages_json)
                media = json.loads(media_json)

                cursor.execute(
                    "UPDATE posts SET status = 'processing' WHERE id = ?",
                    (post_id,)
                )
                conn.commit()

            success = True
            for page_id in pages:
                page_access_token = get_page_access_token(page_id, selected_token)
                if not page_access_token:
                    logging.error(f"فشل الحصول على token للصفحة {page_id} باستخدام {selected_token[:10]}...")
                    # محاولة بتوكن جديد إذا فشل الأول
                    page_access_token = get_page_access_token(page_id)
                    if not page_access_token:
                        logging.error(f"فشل نهائي في الحصول على token للصفحة {page_id}")
                        success = False
                        continue

                media_ids = []
                for media_path in media:
                    try:
                        with open(media_path, "rb") as file:
                            response = requests.post(
                                f"https://graph.facebook.com/v18.0/{page_id}/photos",
                                files={"source": file},
                                data={"access_token": page_access_token, "published": "false"}
                            )
                            if response.status_code != 200:
                                logging.error(f"فشل رفع الوسائط للصفحة {page_id}: {response.text}")
                                success = False
                                break
                            media_ids.append({"media_fbid": response.json()["id"]})
                    except Exception as e:
                        logging.error(f"خطأ في تحميل الملف للصفحة {page_id}: {str(e)}")
                        success = False
                        break

                if not success:
                    break

                payload = {
                    "message": message,
                    "access_token": page_access_token,
                    "attached_media": json.dumps(media_ids) if media_ids else None
                }
                try:
                    response = requests.post(
                        f"https://graph.facebook.com/v18.0/{page_id}/feed",
                        data={k: v for k, v in payload.items() if v is not None}
                    )
                    if response.status_code != 200:
                        logging.error(f"فشل النشر على الصفحة {page_id}: {response.text}")
                        success = False
                    else:
                        logging.info(f"تم النشر بنجاح على الصفحة {page_id}")
                except Exception as e:
                    logging.error(f"خطأ في النشر على الصفحة {page_id}: {str(e)}")
                    success = False

            with sqlite3.connect(DB_FILE) as conn:
                cursor = conn.cursor()
                new_status = 'posted' if success else 'scheduled'
                cursor.execute(
                    "UPDATE posts SET status = ? WHERE id = ?",
                    (new_status, post_id)
                )
                conn.commit()
                logging.info(f"تم تحديث حالة المنشور {post_id} إلى {new_status}")

            time.sleep(10)

    except Exception as e:
        logging.error(f"خطأ عام: {str(e)}")
    finally:
        is_thread_running = False

# بدء الخدمة
if not is_thread_running:
    thread = Thread(target=publish_scheduled_posts, daemon=True)
    thread.start()

if __name__ == '__main__':
    app.run(debug=True)